/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010-2012 University of Bonn. All rights reserved.
 *
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * RemoveShapeAction.cpp
 *
 *  Created on: Oct 2, 2012
 *      Author: ankele
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "CodePatterns/MemDebug.hpp"

#include "Shapes/ShapeFactory.hpp"
#include "Shapes/ShapeRegistry.hpp"
#include "CodePatterns/Log.hpp"
#include "CodePatterns/Verbose.hpp"

#include <iostream>
#include <string>

#include "RemoveShapeAction.hpp"

using namespace MoleCuilder;

// and construct the stuff
#include "RemoveShapeAction.def"
#include "Action_impl_pre.hpp"
/** =========== define the function ====================== */
ActionState::ptr ShapeRemoveShapeAction::performCall() {
  LOG(1, "Removing shapes from ShapeRegistry.");

  std::vector<Shape*> selectedShapes = ShapeRegistry::getInstance().getSelectedShapes();
  std::vector<Shape> old_shapes;

  for (unsigned int i=0;i<selectedShapes.size();i++){
    old_shapes.push_back(*selectedShapes[i]);
    ShapeRegistry::getInstance().removeShape(selectedShapes[i]->getName());
  }

  return ActionState::ptr(new ShapeRemoveShapeState(old_shapes, params));
}

ActionState::ptr ShapeRemoveShapeAction::performUndo(ActionState::ptr _state) {
  ShapeRemoveShapeState *state = assert_cast<ShapeRemoveShapeState*>(_state.get());

  //LOG(1, "INFO: Removing '" << state->old_name << "' from ShapeRegistry.");

  for (unsigned int i=0;i<state->old_shapes.size();i++){
    ShapeRegistry::getInstance().addShape(state->old_shapes[i]);
    ShapeRegistry::getInstance().selectShape(state->old_shapes[i].getName());
  }

  return ActionState::ptr(_state);
}

ActionState::ptr ShapeRemoveShapeAction::performRedo(ActionState::ptr _state){
//  ShapeRemoveShapeState *state = assert_cast<ShapeRemoveShapeState*>(_state.get());

  std::vector<Shape*> selectedShapes = ShapeRegistry::getInstance().getSelectedShapes();
  for (unsigned int i=0;i<selectedShapes.size();i++)
    ShapeRegistry::getInstance().removeShape(selectedShapes[i]->getName());

  return ActionState::ptr(_state);
}

bool ShapeRemoveShapeAction::canUndo() {
  return true;
}

bool ShapeRemoveShapeAction::shouldUndo() {
  return true;
}
/** =========== end of function ====================== */
